require("Mesh_Builder")
require("Shaders")
require("Sound")
require("Table_Util")

local mesh_resource_builders =
{
	ship = function(mb)
		mb:add_supershape(50, 50,
			{ n1 = 0.15, n2 = 2, n3 = 0.9, m = 3, a = 1.00, b = 1.0},
			{ n1 = 3.1, n2 = 1.1, n3 = 0.9, m = 6, a = 20, b = 0.8})
		mb:scale(1)
		mb:rot_xyz(math.pi / 2, math.pi / 2, 0)
		end,

	cube = function(mb)
		mb:add_cube()
		end,
		
	ball = function(mb)
		mb:add_cube()
		mb:smooth()
--		mb:smooth()
		end,
		
	star = function(mb)
		mb:add_supershape(50, 50,
			{ n1 = 0.1, n2 = 1.7, n3 = 1.7, m = 5, a = 1, b = 1},
			{ n1 = 0.3, n2 = 0.5, n3 = 0.5, m = 1, a = 1, b = 1})
		mb:scale(3)
		mb:rot_xyz(math.pi / 2, 0, 0)
		end,

	cube_smooth = function(mb)
		mb:add_supershape(50, 50,
			{ n1 = 10, n2 = 10, n3 = 10, m = 4, a = 1, b = 1},
			{ n1 = 10, n2 = 10, n3 = 10, m = 4, a = 1, b = 1})
		end,
		
	diamond = function(mb)
		mb:add_supershape(20, 8,
			{ n1 = 1, n2 = 1, n3 = 1, m = 4, a = 1, b = 1},
			{ n1 = 1, n2 = 1, n3 = 1, m = 4, a = 1, b = 1})
		end,

	butterfly = function(mb)
		mb:add_supershape(50, 50,
			{ n1 = 0.15, n2 = 4.0, n3 = 0.8, m = 3, a = 1, b = 1},
			{ n1 = 2.1, n2 = 1.1, n3 = 0.7, m = 2, a = 5.1, b = 0.4})
		mb:scale(1)
--		mb:rot_xyz(0, math.pi / 2, 0)
		mb:rot_xyz(math.pi / 2, math.pi / 2, 0)
		end,

	twin_gun = function(mb)
		mb:add_supershape(20, 8,
			{ n1 = 3, n2 = 2.5, n3 = 0.2, m = 1.5, a = 2, b = 1},
			{ n1 = 2.5, n2 = 3.3, n3 = 0.2, m = 17, a = 0.2, b = 0.5})
		mb:rot_xyz(0, math.pi / 2, 0)
		end,
		
	menu_background = function(mb)
		mb:add_supershape(100, 100,
			{ n1 = -0.2, n2 = 1.7, n3 = 1.7, m = 5, a = 1, b = 1},
			{ n1 = 0.7, n2 = 0.5, n3 = 0.5, m = 10, a = 1, b = 1})
		mb:scale(3)
		mb:rot_xyz(math.pi / 2, 0, 0)
		end,
}

local shader_resource_builders =
{
	shader_normal = { shaders.vs_normal, shaders.fs_normal },
	shader_bullet = { shaders.vs_bullet, shaders.fs_normal },
	shader_enemy = { shaders.vs_enemy, shaders.fs_normal },

	shader_normal_line = { shaders.vs_normal_line, shaders.fs_normal },
}

local material_resource_builders =
{
	material_normal = function()
		local app = appearance_create()
		local shader = resources:get("shader_normal")
		appearance_set_shader(app, shader)
		return { appearance = app, shader = shader, }
		end,
		
	material_bullet = function()
		local app = appearance_create()
		local shader = resources:get("shader_bullet")
		appearance_set_shader(app, shader)
		appearance_set_blending(app, BLEND_ALPHA_ADD)
		return { appearance = app, shader = shader, }
		end,

	material_enemy = function()
		local app = appearance_create()
		local shader = resources:get("shader_enemy")
		appearance_set_shader(app, shader)
		return { appearance = app, shader = shader, }
		end,
		
	material_menu_bg = function()
		local app = appearance_create()
		local shader = resources:get("shader_normal")
		appearance_set_shader(app, shader)
		return { appearance = app, shader = shader, }
		end,
}

sound_resource_builders =
{
	sound_shoot = "Data/Audio/shoot.wav",
	sound_player_death = "Data/Audio/player_death.wav",
	sound_enemy_hit = "Data/Audio/enemy_hit.wav",
	sound_enemy_death = "Data/Audio/enemy_death.wav",
}

-- Collect the different mesh resource builders
resource_builders = {}

for k,v in pairs(mesh_resource_builders) do
	local base_builder = v
	local function builder()
		mesh_builder:clear()
		base_builder(mesh_builder)
		return mesh_builder:build_mesh()
	end
	
	resource_builders[k] = builder
end

for k,v in pairs(shader_resource_builders) do
	local vshader = v[1]
	local fshader = v[2]
	local function builder()
		return shader_create(vshader, fshader)
	end
	
	resource_builders[k] = builder
end

table_combine(resource_builders, material_resource_builders)

for k,v in pairs(sound_resource_builders) do
	local sound_name = v
	local function builder()
		return new_sound(sound_name)
	end
	
	resource_builders[k] = builder
end

-- Create the resources object
resources =
{
	built_resources = {},

	update_load = function(this)
		-- Any meshes not yet generated?
		local build_left = 3
		
		for k, v in pairs(resource_builders) do
			if this.built_resources[k] == nil then
				this:get(k)
				
				-- Built enough this frame?
				build_left = build_left - 1
				if build_left <= 0 then
					return true
				end
			end
		end
		
		return false
		end,
	
	get = function(this, res_name)
		local data = this.built_resources[res_name]
		
		if res_name == nil then
			return nil
		end
		
		-- Resource not found; can we build it?
		if data == nil then
			local builder = resource_builders[res_name]
			if builder ~= nil then
				-- Build the resource
				data = builder()
				
				-- Store the built resource
				this.built_resources[res_name] = data
			end
		end
		
		return data
		end,
}
